/****************************************************  Common Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are common to all Wavefront products.

	Rev History: 2-20-2014:		Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Common Functions  *****************************************************/
#include <string.h>

#include "adapter.h"
#include "wavefront.h"
#include "common_functions.h"
#include "device_functions.h"

// Close the Red Rapids device on the PCI bus.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long BusClose(s_Adapter *p_HW)
{
	// Close the assigned device.
	return(Adapter_Close(p_HW));
}

// Open the Red Rapids device on the PCI bus.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long BusOpen(s_Adapter *p_HW)
{
	// Set all of the global interrupt mask register values in the hardware handle.
	// Note: This product does not produce interrupts, so an unused address is selected.
	p_HW->GlobalIntMask.BAR = 2;
	p_HW->GlobalIntMask.Offset = 0xFF;
	p_HW->GlobalIntMask.Disable = 0x0ull;
	// Open the assigned device.
	return(Adapter_Open(p_HW));
}

// Report the current status of all clocks.
// Returns the following status codes:
// 0: External sample clock active.
// 1: Sample clock locked to internal reference.
// 2: Sample clock locked to external reference.
// 3: Sample clock is not locked.
unsigned long ClockStatus(s_Adapter *p_HW)
{
	unsigned Value = 0;
	unsigned long Status = 0;
	
	// Read the sample clock status register.
	Read(p_HW, BAR2, COMMAND_STATUS, &Value);

	if (Value & 0x00000010)
		// External sample clock active.
		Status = 0;
	else
		if (!(Value & 0x00000008))
			//Sample clock not locked.
			Status = 3;
		else
			if (Value & 0x00000020)
				// Sample clock locked to internal reference.
				Status = 1;
			else
				// Sample clock locked to external reference.
				Status = 2;

	return(Status);
}

// Report the firmware revision.
// Returns the firmware revision.
unsigned long FirmwareRevision(s_Adapter *p_HW)
{
	unsigned Value = 0;
	
	Read(p_HW, BAR2, IDENTIFICATION, &Value);
	
	return((unsigned long)(Value & 0x00000007));
}

// Pause for the specified time in microseconds.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long MicrosecondTimer(unsigned long Time)
{
	// Pause for the specified time.
	return(Adapter_uSleep(Time));
}

// Perform a 32-bit register read from the device at the specified BAR and address offset.
// Note: This function applies to both 32-bit and 64-bit operating systems.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long Read(s_Adapter *p_HW, unsigned long Bar, unsigned long Off, unsigned *p_Value)
{
	unsigned long Status = 0;

	// Read the requested offset and store the result in Value.
	Status = Adapter_Read32(p_HW, Bar, Off, p_Value);
	return(Status);
}

// Program a device through a serial interface port using software commands.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long SoftSerial(s_Adapter *p_HW, unsigned ChipSelect, unsigned Payload, size_t PayloadSize)
{
	unsigned long Status = 0;
	size_t bitcnt;

	// Disable all chip selects and set the clock low.
	Status += Write(p_HW, BAR2, SERIAL_CONFIG, 0x78);

	// Pause to meet the worst case interface timing constraints.
	MicrosecondTimer(50);

	for (bitcnt = 0; bitcnt < PayloadSize; bitcnt++)
	{
		// Toggle the clock low with chip select enabled and load the next data bit.
		Status += Write(p_HW, BAR2, SERIAL_CONFIG, ChipSelect << 3 | (((Payload >> (PayloadSize-bitcnt-1)) & 0x1) << 1));
		// Pause to meet the worst case interface timing constraints.
		MicrosecondTimer(50);
		// Toggle the clock high with chip select enabled while holding the current data bit.
		Status += Write(p_HW, BAR2, SERIAL_CONFIG, ChipSelect << 3 | 0x4 | (((Payload >> (PayloadSize-bitcnt-1)) & 0x1) << 1));
		// Pause to meet the worst case interface timing constraints.
		MicrosecondTimer(50);
	}

	// Toggle the clock low with the chip select held active, the data value does not matter.
	Status += Write(p_HW, BAR2, SERIAL_CONFIG, ChipSelect << 3);
	// Pause to meet the worst case interface timing constraints.
	MicrosecondTimer(50);

	// Disable all chip selects and set the clock low.
	Status += Write(p_HW, BAR2, SERIAL_CONFIG, 0x78);

	// Pause to meet the worst case interface timing constraints.
	MicrosecondTimer(50);

	// Return an error code if the program operation did not complete.
	if (Status)
		return(1);
	else
		return(0);
}

// Perform a 32-bit target PCI write.
// Note: This function applies to both 32-bit and 64-bit operating systems.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long Write(s_Adapter *p_HW, unsigned long Bar, unsigned long Off, unsigned Value)
{
	// Write the contents of Value in the specified offset.
	return(Adapter_Write32(p_HW, Bar, Off, Value));
}