/****************************************************  Device Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are used to initialize the hardware specific to each SigFPGA model.

	Rev History: 1-10-2014:		Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Device Functions  *****************************************************/

#include <stdlib.h>

#include "adapter.h"
#include "wavefront.h"
#include "common_functions.h"
#include "device_functions.h"

// Enable or disable the direct RF mode.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long DirectRFEnable(s_Adapter *p_HW, unsigned long Enable)
{
	unsigned Value = 0;
	unsigned long Status = 0;
	
	// Read the command/status register to modify the current settings.
	Read(p_HW, BAR2, COMMAND_STATUS, &Value);

	if (Enable)
		// Enable the direct RF by setting the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value | 0x4);
	else
		// Disable the direct RF divider by clearing the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value & ~0x4);

	return(Status);
}

// Enable or disable the carrier frequency divider.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long FreqDiv2Enable(s_Adapter *p_HW, unsigned long Enable)
{
	unsigned Value = 0;
	unsigned long Status = 0;
	
	// Read the command/status register to modify the current settings.
	Read(p_HW, BAR2, COMMAND_STATUS, &Value);

	if (Enable)
		// Enable the frequency divider by setting the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value | 0x1);
	else
		// Disable the frequency divider by clearing the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value & ~0x1);

	return(Status);
}

// Perform a serial write transaction to the HMC624LP4 attenuator chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LoadAttenuator(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Data)
{
	// Write data to the HMC624LP4 register.
	return(SoftSerial(p_HW, ChipSelect, Data, 6));
}

// Perform a serial write transaction to the ASY-817-034 frequency synthesizer.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LoadFrequency(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Frequency)
{
	unsigned long Status = 0;

	// Pause to meet the 1.5 ms setup constraint.
	MicrosecondTimer(1500);

	// Write the requested frequency to the "N Counter" register.
	Status += SoftSerial(p_HW, ChipSelect, Frequency << 1 | 0x1, 24);

	// Pause to meet the 1.5 ms setup constraint.
	MicrosecondTimer(1500);

	// Set the "R Counter" register to 10 MHz.
	Status += SoftSerial(p_HW, ChipSelect, 0xA << 1, 24);

	return(Status);
}

// Perform a serial write transaction to the LTC1661 DAC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LTC1661Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	// Write data to the LTC1661 register at the requested address.
	return(SoftSerial(p_HW, ChipSelect, Address << 12 | Data << 2, 16));
}

// Enable or disable the reference clock output.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long RefClkOutEnable(s_Adapter *p_HW, unsigned long Enable)
{
	unsigned Value = 0;
	unsigned long Status = 0;
	
	// Read the command/status register to modify the current settings.
	Read(p_HW, BAR2, COMMAND_STATUS, &Value);

	if (Enable)
		// Enable the reference clock output by setting the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value | 0x2);
	else
		// Disable the reference clock output by clearing the command bit.
		Status += Write(p_HW, BAR2, COMMAND_STATUS, Value & 0xFD);

	return(Status);
}

