/*****************************************************************************************
	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following code demonstrates the operation of the Red Rapids Wavefront PCI Express product.

	Note:	The application can be applied to the maximum number of Red Rapids devices (MAX_NUM_DEVICES) supported, but only
			a single device (DEV0) is demonstrated.

	Rev History: 2-20-2014:		Adopted the generic Channel Adapter driver that is used on all PCI products.
				 2-19-2009:		Changed memmap definition from INT_LO_ACTIVE to INT_LO_LOCKED, changed help printout message.
				 2-13-2009:		Require R value to be set when N value is set. Corrections to BDAC functions.
				 11-12-2008:	Inital release.
	
	Copyright 2014, Red Rapids, Inc.

******************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "adapter.h"				// Adpater device driver common to all Red Rapids PCI Express products.
#include "wavefront.h"				// Wavefront application.
#include "common_functions.h"		// Functions common to all Signal Stream hardware and channel types.
#include "device_functions.h"		// Functions that apply to specific devices on the Signal Stream hardware.
#include "device_defaults.h"		// Default device configuration values.

int main()
{
	s_Adapter *p_HW[MAX_NUM_DEVICES];			// Hardware handle to the Red Rapids device.
	unsigned long devcnt = 0;					// Variable to index device numbers in loops.
	unsigned long Handle[MAX_NUM_DEVICES];		// Array to store hardware handle allocation status for each device (success = 0, failure = 1).
	unsigned long Status = 0;					// Variable to store status returned by function calls.
	size_t index = 0;							// Variable to index array elements in loops.

	printf("******************************* Wavefront *******************************\n");

	//////////////////////////////////////////////  Initialize Hardware Handle  //////////////////////////////////////////////

	printf("\nInitializing structures...\n");

	// Initialize the hardware handle for each device.
	for (devcnt = 0; devcnt < MAX_NUM_DEVICES; devcnt++)
	{
		// Allocate memory for the structure.
		p_HW[devcnt] = (s_Adapter*) malloc(sizeof(s_Adapter));
		if (p_HW[devcnt] == NULL)
		{
			// This is a fatal error because there is no hardware handle available.
			printf("\tERROR: Unable to allocate memory for the device #%lu hardware handle.\n", devcnt);
			Status++;
			// Mark this handle as a failed allocation.
			Handle[devcnt] = 1;
		}
		else
		{
			// Mark this handle as a successful allocation.
			Handle[devcnt] = 0;
			// Initialize each member of the structure.
			Adapter_Zero(p_HW[devcnt]);
			// Set a unique device number for each hardware handle.
			p_HW[devcnt]->DevNum = devcnt;
			// Set the user defined pointer to null since this device does not produce interrupts.
			p_HW[devcnt]->User = NULL;
		}
	}

	//////////////////////////////////////////////////  Configure Hardware  //////////////////////////////////////////////////
	if (!Status)
	{
		printf("\nOpening device #%u on the host...\n", DEV0);

		// Open the Red Rapids device on the bus and only continue if successful.
		if (!BusOpen(p_HW[DEV0]))
		{
			// Print the firmware revision to the screen.
			printf("\tFirmware revision: %01lX\n", FirmwareRevision(p_HW[DEV0]));

			// Report the local oscillator status returned from the hardware.
			switch (ClockStatus(p_HW[DEV0]))
			{
				case 0: {printf("\tExternal local oscillator active.\n"); break;}
				case 1: {printf("\tInternal local oscillator locked to internal reference.\n"); break;}
				case 2: {printf("\tInternal local oscillator locked to external reference.\n"); break;}
				case 3: {printf("\tLocal oscillator is not locked.\n"); Status++; break;}
				default: {printf("\tERROR: Local oscillator status unknown.\n"); Status++; break;}
			}

			// Set the carrier frequency.
			Status += LoadFrequency(p_HW[DEV0], SYNTHESIZER, M150Default.Frequency);

			// Enable or disable the carrier frequency divider.
			Status += FreqDiv2Enable(p_HW[DEV0], M150Default.FreqDiv2);

			// Set the attenuator value.
			Status += LoadAttenuator(p_HW[DEV0], ATTENUATOR, M150Default.Attenuator);

			// Enable or disable the reference clock output.
			Status += RefClkOutEnable(p_HW[DEV0], M150Default.RefClkOut);

			// Enable or disable the direct RF mode.
			Status += DirectRFEnable(p_HW[DEV0], M150Default.DirectRF);
	
			// Configure the trim DACs.
			for (index = 0; index < 3; index++)
			{
				Status += LTC1661Write(p_HW[DEV0], TRIM_DAC_I, M150Default.TrimDACI.Payload[index][0], M150Default.TrimDACI.Payload[index][1]);
				Status += LTC1661Write(p_HW[DEV0], TRIM_DAC_Q, M150Default.TrimDACQ.Payload[index][0], M150Default.TrimDACQ.Payload[index][1]);
			}
	
			// Close the Red Rapids device on the bus.
			Status = BusClose(p_HW[DEV0]);
		}
		else
			printf("\nERROR: Terminating execution because device could not be opened.\n");
	}
	else
		printf("\nERROR: Terminating execution due to hardware handle intialization errors.\n");

	/////////////////////////////////////////////////  Free Allocated Memory /////////////////////////////////////////////////

	for (devcnt = 0; devcnt < MAX_NUM_DEVICES; devcnt++)
		if (!Handle[devcnt]) free(p_HW[devcnt]);

return 0;

}

